/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "mine.hpp"
#include "nullable.hpp"
#include "rule.hpp"
#include "rulestats.hpp"
#include <vector>

/// A storage for the best rules found by the data mining algorithm
class BestRules
{
	private: size_t f_capacity;
    private: std::vector<Rule> f_items;
    private: std::set<std::int64_t> f_hashes;

	public: BestRules(size_t _capacity = 0)
        : f_capacity(_capacity)
	{
	}

	public: const std::vector<Rule>& items() const
	{
	    return f_items;
    }

	public: bool shouldInsertRule(const double goodness) const;
	/// Where should we insert a rule with a specific goodness?
	/// Returns nullptr if should not insert
	private: Nullable<size_t> getRuleInsertIndex(const double goodness) const;
	/// Tries to insert a rule to the list of best rules
	public: void insertRule(const Rule &rule);
	/// Tries to insert all the rules from another BestRules object
	public: void merge(const BestRules& other);
};
