/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "bits.hpp"

char LookupPopCount::popCountTable[0x10000u];

void LookupPopCount::init()
{
    for (size_t i = 0; i < 0x10000u; ++i)
        popCountTable[i] = SimplePopCount::get(i);
}

size_t LookupPopCount::get(uint16_t x)
{
    return static_cast<size_t>(popCountTable[x]);
}

size_t LookupPopCount::get(uint32_t x)
{
    return static_cast<size_t>(popCountTable[x & UINT32_C(0xffff)]) +
        static_cast<size_t>(popCountTable[x >> 16]);
}

size_t LookupPopCount::get(uint64_t x)
{
    // facilitate CPU piping
    const size_t a = static_cast<size_t>(popCountTable[x & UINT64_C(0xffff)]);
    const size_t b = static_cast<size_t>(popCountTable[(x >> 16) & UINT64_C(0xffff)]);
    const size_t c = a + static_cast<size_t>(popCountTable[(x >> 32) & UINT64_C(0xffff)]);
    const size_t d = b + static_cast<size_t>(popCountTable[x >> 48]);
    return c + d;
}

size_t AdvancedPopCount::get(uint32_t x)
{
    x -= (x >> 1) & UINT32_C(0x55555555);
    x = (x & UINT32_C(0x33333333)) + ((x >> 2) & UINT32_C(0x33333333));
    x = (x + (x >> 4)) & UINT32_C(0x0f0f0f0f);
    x += x >> 8;
    x += x >> 16;
    return x & 0x3f;
}

size_t AdvancedPopCount::get(uint64_t x)
{
    x -= (x >> 1) & UINT64_C(0x5555555555555555);
    x = (x & UINT64_C(0x3333333333333333)) + ((x >> 2) & UINT64_C(0x3333333333333333));
    x = (x + (x >> 4)) & UINT64_C(0x0f0f0f0f0f0f0f0f);
    x += x >> 8;
    x += x >> 16;
    x += x >> 32;
    return x & 0x7f;
}

void DefaultPopCount::init()
{
    LookupPopCount::init();
}

/*
// Unfortunately we don't have adequate SSE instructions for this.
Bits128 Bits128::operator<<(size_t y) const
{
    if (y >= 128) return Zeros();

    Bits128 result = *this;
    if (y >= 64)
    {
        result.data64[1] = result.data64[0];
        result.data64[0] = 0;
        y &= 63;
    }

    result.data64[1] = (result.data64[1] << y) | (result.data64[0] >> (64-y));
    result.data64[0] <<= y;

    return result;
}

template <>
Bits128 Zeros<Bits128>()
{
    return Bits128::Zeros();
}

template <>
Bits128 Ones<Bits128>()
{
    return Bits128::Ones();
}

template <>
size_t SimplePopCount::get<Bits128>(Bits128 x)
{
    return get(x.data64[0]) + get(x.data64[1]);
}
*/
