/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "bestrules.hpp"
#include "concurrent_stream.hpp"
#include "mine.hpp"
#include "params.hpp"
#include "rule.hpp"
#include "table.hpp"

/// WARNING: This class has reference fields.
class RuleMiner
{
	// INPUT
	public: const Mine& mine;
	public: const Params& params;
	public: const ConcurrentStream& logger;
    private: double eValueFactor;

	public: RuleMiner() = delete;
	public: RuleMiner(const Mine& _mine, const Params& _params, const ConcurrentStream& _logger);

    /// Gets the indexes of those mine columns which can appear on the LHS of a rule
    /// as specified by 'params'.
    public: std::vector<size_t> get_lhsColumnIndexes() const;
    /// Gets the indexes of those mine columns which can appear on the RHS of a rule
    /// as specified by 'params'
    public: std::vector<size_t> get_rhsColumnIndexes() const;

    /// Runs the data mining algorithm for the given RHS and returns the best rules found.
    /// Parameter: the index of the mine column which should be on the RHS of the returned rules.
    /// This column must be an element of rhsColumns
	public: BestRules operator()(size_t rhsIndex) const;
};
