/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "logicalexpr.hpp"
#include <string>
#include "table.hpp"
#include <vector>
#include "xml.hpp"

class Rule
{
	public: ConjunctiveNormalForm lhs;
	public: size_t rhsColumnIndex;
	public: RuleStats stats;

	public: Rule()
	{
	}

	public: Rule(const ConjunctiveNormalForm &_lhs, const size_t _rhsColumnIndex, const RuleStats &_stats)
        : lhs(_lhs), rhsColumnIndex(_rhsColumnIndex), stats(_stats)
	{
	}

	/// Logically equivalent rules should return the same hash
	public: std::int64_t getHash() const;
	public: std::string toString(const Mine &mine) const;
    public: void toXml(XmlWriter& xml, const Mine &mine) const;

	/// simplification: trying to delete elementary clauses such that
	/// the overall goodness of the rule does not decrease too much with one removal,
	/// and the rule is still better than minStats.
	/// If there are elementary clauses, by whose removal the rule statistics do not change much with respect to
	/// the ORIGINAL universe before the removal, then these clauses must also be removed.
	public: bool canSimplify(const Mine &mine, const RuleStats &minStats,
                         const double eValueFactor, const double allowedGoodnessDecrease) const;
};
