/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <cstddef>
#include <cstdint>
#include <iostream>
#include <limits>
#include "logicalexpr.hpp"
#include <string>
#include <vector>
#include "xml.hpp"

/// The performance of a rule.
class RuleStats
{
	// Universe: where no fields contained by X or Y are N/A

	// *** Basic fields
	public: size_t universe;
	public: size_t supportLhs, supportRhs; // supp(X), supp(Y)
	public: size_t support; // supp(x U y)

	// *** Transient fields
	//     (these can be calculated from the basic fields)
	public: double confidence; // supp(x U y) / supp(X)
	public: double lift; // P(y|x) / P(y)
	public: double leverage; // P(x and y) - P(X)*P(Y)

    // values for the chi^2 test
    public: double pValue;
    public: double eValue;

    public: RuleStats() { clear(); }

    public: void clearBasicFields()
    {
		universe = 0;
		supportLhs = 0;
		supportRhs = 0;
		support = 0;
    }

	public: void clear()
	{
        clearBasicFields();

		confidence = 0;
		lift = 0;
		leverage = 0;
		pValue = 1;
		eValue = std::numeric_limits<double>::max();
	}

	public: double goodness() const
	{
		return lift; // we are searching for rules with best lift
	}

    /// \param eValueFactor The E-value/p-value ratio (i.e. the database size)
	public: void calculateTransientValues(const double eValueFactor);
	public: bool atLeastAsGoodAs(const RuleStats &other) const;
	public: std::string toString() const;
	public: void toXml(XmlWriter& xml) const;
};
