/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "table.hpp"

#include <algorithm>
#include <cstdlib>
#include <fstream>
#include <iostream>

void Table::load(const std::string& fileName)
{
    std::ifstream f(fileName);
    std::string line;

    clear();

    std::getline(f, line);
    const size_t count_semicolon = std::count(line.begin(), line.end(), ';');
    const size_t count_comma = std::count(line.begin(), line.end(), ',');
    const char sep = (count_semicolon > count_comma ? ';' : ',');
	const std::vector<std::string> colNames = csvSplit(line, sep);
	f_columns.reserve(colNames.size());
    for (const std::string& colName : colNames)
        f_columns.emplace_back(strip(colName));

    while (f)
    {
        std::getline(f, line);
        if (line.size() == 0) continue; // empty line
        std::vector<std::string> cells = csvSplit(line, sep);

        std::string currentRow(columns.size(), CHAR_NA);
        for (size_t i = 0, iMax = std::min(columns.size(), cells.size()); i < iMax; ++i)
        {
            const std::string& cell = at(cells, i);
            auto range = stripToRange(cell);

            char c = (range.first == range.second ? CHAR_NA : *range.first);
            at(currentRow, i) = c;

            if (c != CHAR_NA)
            {
                TableColumn &col = at(f_columns, i);
                col.universe++;
                col.values.insert(c);
                if (range.second - range.first >= 2)
                    col.hadLongCells = true;
            }
        }
        f_rows.push_back(currentRow);
    }
}

bool Table::validColumnName(const std::string& name)
{
    if (name.empty()) return false;
    for (char c : name)
    {
        bool ok = ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c >= '0' && c <= '9')
                   || c == '.' || c == '_' || c == '[' || c == ']');
        if (!ok) return false;
    }
    return true;
}
