/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "utils.hpp"

#include <algorithm>
#include <functional>
#include <stdexcept>

std::int64_t getHash(const std::string &s)
{
    return getHash(s.c_str());
}

bool isWhitespace(char c)
{
    return c <= ' ';
}

bool isNonWhitespace(char c) { return !isWhitespace(c); }

std::int64_t parseInt(const std::string &s)
{
    return parseInt(s.c_str());
}

std::vector<std::string> split(const std::string &s, const char separator, const EmptyEntryAction a)
{
    return split(s, std::bind(std::equal_to<char>(), std::placeholders::_1, separator), a);
}

std::vector<std::string> csvSplit(const std::string &s, const char separator)
{
    std::vector<std::string> result;
    if (!s.empty())
    {
        std::string cell;
        bool quoted = false;
        for (size_t i = 0; i < s.size(); ++i)
        {
            char c = at(s, i);
            char cNext = (i+1 < s.size() ? at(s, i+1) : ' ');
            if (c == '"')
            {
                if (cell.empty())
                    quoted = true;
                else if (quoted)
                {
                    if (cNext == '"')
                    {
                        cell += c;
                        ++i;
                    }
                    else
                        quoted = false;
                }
                else
                    cell += c;
            }
            else if (c == separator && !quoted)
            {
                result.push_back(cell);
                cell.clear();
            }
            else
                cell += c;
        }
        result.push_back(cell);
    }
    return std::move(result);
}

std::pair<std::string::const_iterator,std::string::const_iterator>
    stripToRange(const std::string &s)
{
    std::pair<std::string::const_iterator,std::string::const_iterator> p;
    p.first = std::find_if(s.begin(), s.end(), isNonWhitespace);
    if (p.first == s.end())
        p.second = s.end();
    else p.second = std::find_if(s.rbegin(), s.rend(), isNonWhitespace).base();
    return p;
}

std::string strip(const std::string &s)
{
    auto r = stripToRange(s);
    return std::string(r.first, r.second);
}
